#ifndef GD_H
#define GD_H 1

#if defined(__STDC__) || defined(__cplusplus)
#  define __ANSI_PROTO(x)       x
#else
#  define __ANSI_PROTO(x)       ()
#endif



/* gd.h: declarations file for the gifdraw module.

	Written by Tom Boutell, 5/94.
	Copyright 1994, Cold Spring Harbor Labs.
	Permission granted to use this code in any fashion provided
	that this notice is retained and any alterations are
	labeled as such. It is requested, but not required, that
	you share extensions to this module with us so that we
	can incorporate them into new versions. */

/* stdio is needed for file I/O. */
#include <stdio.h>

/* This can't be changed, it's part of the GIF specification. */

#define gdMaxColors 256

/* Image type. See functions below; you will not need to change
	the elements directly. Use the provided macros to
	access sx, sy, the color table, and colorsTotal for 
	read-only purposes. */

typedef struct gdImageStruct {
	unsigned char ** pixels;
	int sx;
	int sy;
	int colorsTotal;
	int red[gdMaxColors];
	int green[gdMaxColors];
	int blue[gdMaxColors]; 
	int open[gdMaxColors];
	int transparent;
	int *polyInts;
	int polyAllocated;
	struct gdImageStruct *brush;
	struct gdImageStruct *tile;	
	int brushColorMap[gdMaxColors];
	int tileColorMap[gdMaxColors];
	int styleLength;
	int stylePos;
	int *style;
	int interlace;
} gdImage;

typedef gdImage * gdImagePtr;

typedef struct {
	/* # of characters in font */
	int nchars;
	/* First character is numbered... (usually 32 = space) */
	int offset;
	/* Character width and height */
	int w;
	int h;
	/* Font data; array of characters, one row after another.
		Easily included in code, also easily loaded from
		data files. */
	char *data;
} gdFont;

/* Text functions take these. */
typedef gdFont *gdFontPtr;

/* For backwards compatibility only. Use gdImageSetStyle()
	for MUCH more flexible line drawing. Also see
	gdImageSetBrush(). */
#define gdDashSize 4

/* Special colors. */

#define gdStyled (-2)
#define gdBrushed (-3)
#define gdStyledBrushed (-4)
#define gdTiled (-5)

/* NOT the same as the transparent color index.
	This is used in line styles only. */
#define gdTransparent (-6)

/* Functions to manipulate images. */

gdImagePtr gdImageCreate __ANSI_PROTO((int sx, int sy));
gdImagePtr gdImageCreateFromGif __ANSI_PROTO((FILE *fd));
gdImagePtr gdImageCreateFromGd __ANSI_PROTO((FILE *in));
gdImagePtr gdImageCreateFromXbm __ANSI_PROTO((FILE *fd));

void gdImageDestroy __ANSI_PROTO((gdImagePtr im));
void gdImageSetPixel __ANSI_PROTO((gdImagePtr im, int x, int y, int color));
int gdImageGetPixel __ANSI_PROTO((gdImagePtr im, int x, int y));
void gdImageLine __ANSI_PROTO((gdImagePtr im, int x1, int y1, int x2, int y2, int color));

/* For backwards compatibility only. Use gdImageSetStyle()
	for much more flexible line drawing. */
void gdImageDashedLine __ANSI_PROTO((gdImagePtr im, int x1, int y1, int x2, int y2, int color));

/* Corners specified (not width and height). Upper left first, lower right
 	second. */
void gdImageRectangle __ANSI_PROTO((gdImagePtr im, int x1, int y1, int x2, int y2, int color));

/* Solid bar. Upper left corner first, lower right corner second. */
void gdImageFilledRectangle __ANSI_PROTO((gdImagePtr im, int x1, int y1, int x2, int y2, int color));
int gdImageBoundsSafe __ANSI_PROTO((gdImagePtr im, int x, int y));
void gdImageChar __ANSI_PROTO((gdImagePtr im, gdFontPtr f, int x, int y, int c, int color));
void gdImageCharUp __ANSI_PROTO((gdImagePtr im, gdFontPtr f, int x, int y, char c, int color));
void gdImageString __ANSI_PROTO((gdImagePtr im, gdFontPtr f, int x, int y, char *s, int color));
void gdImageStringUp __ANSI_PROTO((gdImagePtr im, gdFontPtr f, int x, int y, char *s, int color));

/* Point type for use in polygon drawing. */

typedef struct {
	int x, y;
} gdPoint, *gdPointPtr;

void gdImagePolygon __ANSI_PROTO((gdImagePtr im, gdPointPtr p, int n, int c));
void gdImageFilledPolygon __ANSI_PROTO((gdImagePtr im, gdPointPtr p, int n, int c));

int gdImageColorAllocate __ANSI_PROTO((gdImagePtr im, int r, int g, int b));
int gdImageColorClosest __ANSI_PROTO((gdImagePtr im, int r, int g, int b));
int gdImageColorExact __ANSI_PROTO((gdImagePtr im, int r, int g, int b));
void gdImageColorDeallocate __ANSI_PROTO((gdImagePtr im, int color));
void gdImageColorTransparent __ANSI_PROTO((gdImagePtr im, int color));
void gdImageGif __ANSI_PROTO((gdImagePtr im, FILE *out));
void gdImageGd __ANSI_PROTO((gdImagePtr im, FILE *out));
void gdImageArc __ANSI_PROTO((gdImagePtr im, int cx, int cy, int w, int h, int s, int e, int color));
void gdImageFillToBorder __ANSI_PROTO((gdImagePtr im, int x, int y, int border, int color));
void gdImageFill __ANSI_PROTO((gdImagePtr im, int x, int y, int color));
void gdImageCopy __ANSI_PROTO((gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int srcY, int w, int h));


/* Stretches or shrinks to fit, as needed */
void gdImageCopyResized __ANSI_PROTO((gdImagePtr dst, gdImagePtr src, int dstX, int dstY, int srcX, int srcY, int dstW, int dstH, int srcW, int srcH));
void gdImageSetBrush __ANSI_PROTO((gdImagePtr im, gdImagePtr brush));
void gdImageSetTile __ANSI_PROTO((gdImagePtr im, gdImagePtr tile));
void gdImageSetStyle __ANSI_PROTO((gdImagePtr im, int *style, int noOfPixels));


/* On or off (1 or 0) */
void gdImageInterlace __ANSI_PROTO((gdImagePtr im, int interlaceArg));

/* Macros to access information about images. READ ONLY. Changing
	these values will NOT have the desired result. */
#define gdImageSX(im) ((im)->sx)
#define gdImageSY(im) ((im)->sy)
#define gdImageColorsTotal(im) ((im)->colorsTotal)
#define gdImageRed(im, c) ((im)->red[(c)])
#define gdImageGreen(im, c) ((im)->green[(c)])
#define gdImageBlue(im, c) ((im)->blue[(c)])
#define gdImageGetTransparent(im) ((im)->transparent)
#define gdImageGetInterlaced(im) ((im)->interlace)
#endif
